
/* unnormalized cardinal sine */
float sinc( float x ) {
  if( x==0 ) {
    return 1.0;
  } else {
    return sin(x)/x;
  }
}

/* a bit of syntactic sugar to make it easier to add a vertex at a given (lat,lon) point */
void vertexLL(float la, float lo) {
  vertex( calcX(la, lo), calcY(la, lo) );
}

/* the same, but to draw a line between two pairs of (lat,lon) points */
void lineLL( float la1, float lo1, float la2, float lo2 ) {
  line( calcX(la1,lo1), calcY(la1,lo1), calcX(la2,lo2), calcY(la2,lo2) );
}

/* this determines how the projection will be written on the screen */
void setDisplayName() {
  if( projectionType == WINKEL ) {
    projName = "Winkel Tripel";
  } else if( projectionType == LAMBERT_CYL_EA ) {
    projName = "Lambert E.A. Cylindrical";
  } else if( projectionType == GALL_STEREO ) {
    projName = "Gall Stereographic";
  } else if( projectionType == ALBERS_CON_EA ) {
    projName = "Albers E.A. Conic";
  } else if( projectionType == MERCATOR ) {
    projName = "Mercator";
  } else if( projectionType == KAVRAY ) {
    projName = "Kavrayskiy VII";
  }
}

/* for debugging */
/*  (to make sure I'm not distorting the map when zooming in and out */
void calcAspectRatio() {
  float w0 = calcX(0,-180-longioffset);
  float w1 = calcX(0,180-longioffset);
  float w = w1-w0;
  ///println("width:  ("+nf(w0,4,1)+", "+nf(w1,4,1)+") --> "+nf(w,4,1));
  float h0 = calcY(90,0);
  float h1 = calcY(-90,0);
  float h = h1-h0;
  ///println("height: ("+nf(h0,4,1)+", "+nf(h1,4,1)+") --> "+nf(h,4,1));
  println("ASPECT RATIO == "+nf(w/h,1,4));
}

/* circular indicator that shades from black (midnight) to white (noon) */
/*   note: this is related to actual sunrise/sunset times at all
           it's just a quick visual indication of day/night */ 
void drawSun(Date d) {
  float h = d.getHours();
  float m = d.getMinutes();
  float f = (h+m/60.0)/24.0;
  pushStyle();
  pushMatrix();
  stroke(0);
  float x = map(f, 0, 1, 0, TWO_PI);
  x = sq(map(cos(x), -1, 1, 0, 1));
  x = map(x, 0, 1, 255, 0);
  fill(x);
  translate(20,20);
  ellipse(0,0,8,8);
  popMatrix();
  popStyle();
}

Properties loadCommandLine () {
  Properties props = new Properties();
  for (String arg:args) {
    String[] parsed = arg.split("=", 2);
    if (parsed.length == 2) {
      props.setProperty(parsed[0], parsed[1]);
    }
  }
  return props;
}


/*
See what I said about properties in the main file.
Essentially: ignore this. It's an ugly hack I threw
in at the last second, and should probably just delete.
It's completely irrelevant if you're running this via
the Processing IDE anyway.
*/
void evalProps(Properties props) {
  if( props.getProperty("zoom").equals("usa") ) {
    projectionScale = SCALE_LOWER48;
  } else if( props.getProperty("zoom").equals("nwq") ) {
    projectionScale = SCALE_NORTHWEST;
  } else if( props.getProperty("zoom").equals("glb") ) {
    projectionScale = SCALE_GLOBE;
  }

  if( props.getProperty("proj").equals("winkel") ) {
    projectionType = WINKEL;
  } else if( props.getProperty("proj").equals("kavray") ) {
    projectionType = KAVRAY;
  }
  
  if( props.getProperty("which").equals("both") ) {
    showTweets = true;
    showDonations = true;
  } else if( props.getProperty("which").equals("tweets") ) {
    showTweets = true;
    showDonations = false;
  } else if( props.getProperty("which").equals("money") ) {
    showTweets = false;
    showDonations = true;
  }

  outputName = props.getProperty("out");
}

