/* respond to key presses */
void keyPressed() {
  /* change map projection type */
  if( key == '1' ) {
    projectionType = WINKEL;
    setDisplayName();
  }  else if( key == '2' ) {
    projectionType = KAVRAY;
    setDisplayName();
  } else if( key == '3' ) {
    projectionType = ALBERS_CON_EA; 
    setDisplayName();
  } else if( key == '4' ) {
    projectionType = LAMBERT_CYL_EA;
    setDisplayName();
  } else if( key == '5' ) {
    projectionType = GALL_STEREO;
    setDisplayName();
  } else if( key == '6' ) {
    projectionType = MERCATOR;
    setDisplayName();
  }
  /* change scale of projection */
  else if( key == 'q' ) {
    projectionScale = SCALE_GLOBE;
    longioffset = 0;
  } else if( key == 'w' ) {
    projectionScale = SCALE_NORTHWEST;
    longioffset = 90;
  } else if( key == 'e' ) {
    projectionScale = SCALE_LOWER48;
    longioffset = 96;
  }
  /* speed up or slow down animation */
  else if( key == '+' ) {
    stepSize += 28.8;
    println("stepSize == "+stepSize);
    windowSize = stepSize*20;
  } else if( key == '-' ) {
    if( stepSize > 30 ) {
      stepSize -= 28.8;
      windowSize = stepSize*20;
    }
    println("stepSize == "+stepSize);
  }
  /* switch between 'fade' & 'cumulative' modes */
  else if( key == ' ' ) {
    cumulativeMode = !cumulativeMode;
  }
  /* move center of view */
  else if( key == 'j' ) {
    longioffset += 2.5;
  } else if( key == 'k' ) {
    longioffset -= 2.5;
  } else if( key == 'i' ) {
    latioffset -= 2.5;
  } else if( key == 'm' ) {
    latioffset += 2.5;
  }
  /* zoom in and out */  
  else if( key == '<' ) {
    if( scaleFactor > 0.51 ) {
      scaleFactor -= 0.025;
    }
  } else if( key == '>' ) {
    scaleFactor += 0.025;
  }
  /* for debugging */
  else if( key == 'z' ) {
    calcAspectRatio();
  }
}

void readTweets(String fname) {
  String[] lines = loadStrings(fname);
  String[] oneLine;
  n = lines.length;
  println("Reading "+fname+"  (read " + lines.length + " lines)");
  times = new float[lines.length];
  lat = new float[lines.length];
  lon = new float[lines.length];
  for ( int i = 0; i < lines.length; i++ ) {
    oneLine = split(lines[i], ",");
    times[i] = float(oneLine[0])/1000.0;
    lat[i] = float(oneLine[1]);
    lon[i] = float(oneLine[2]);
    maxTime = max((float)maxTime, times[i]);
    minTime = min((float)minTime, times[i]);
  }
  // subtract out the earliest time
  //   to reduce the mangnitudes we're working with
  //   and simplify some math later on
  for( int i = 0; i < n; i++ ) {
    times[i] = times[i] - (float)minTime;
  }
  duration = maxTime - minTime;
}

void readEye(String fname) {
  String[] lines = loadStrings(fname);
  String[] oneLine;
  println("Reading "+fname+"  (read " + lines.length + " lines)");
  pathTimes = new float[lines.length];
  pathLat = new float[lines.length];
  pathLon = new float[lines.length];
  for ( int i = 0; i < lines.length; i++ ) {
    oneLine = split(lines[i], ",");
    pathTimes[i] = (float(oneLine[0])/1000.0) - (float)minTime;
    pathLat[i] = float(oneLine[1]);
    pathLon[i] = float(oneLine[2]);
  }
}

void readBorders(String fname) {
  String[] lines = loadStrings(fname);
  String[] oneLine;
  println("Reading "+fname+"  (read " + lines.length + " lines)");
  for( int i = 0; i < lines.length; i++ ) {
    oneLine = split(lines[i], "\t");
    if(oneLine.length==1) { // NEW SHAPE
      if( nBorders > 0 ) {
        borders[nBorders-1].endShape();
      }
      nBorders++;
      if( nBorders >= borders.length ) {
        borders = (PShape[])expand(borders);
      }
      borders[nBorders] = new PShape();
      borders[nBorders] = createShape();
      borders[nBorders].beginShape();
    } else {
      PVector newLoc = new PVector( float(oneLine[0]), float(oneLine[1]) );
      if( borders[nBorders].getVertexCount() > epsilon ) {
        PVector prevLoc = borders[nBorders].getVertex(borders[nBorders].getVertexCount()-1);
        newLoc.sub(prevLoc);
        if( newLoc.magSq() > epsilon ) {
          borders[nBorders].vertex(float(oneLine[0]), float(oneLine[1]) );
        }
      } else {
        borders[nBorders].vertex(float(oneLine[0]), float(oneLine[1]) );
      }
    }
  }
  borders[nBorders].endShape();
  nBorders++;
  println("Read " + nBorders + " borders");
  borders = (PShape[])subset(borders,0,nBorders);
}

void readCoasts(String fname) {
  String[] lines = loadStrings(fname);
  String[] oneLine;
  println("Reading "+fname+"  (read " + lines.length + " lines)");
  for( int i = 0; i < lines.length; i++ ) {
    oneLine = split(lines[i], "\t");
    if(oneLine.length==1) { // NEW SHAPE
      if( nCoasts > 0 ) {
        coasts[nCoasts-1].endShape();
      }
      nCoasts++;
      if( nCoasts >= coasts.length ) {
        coasts = (PShape[])expand(coasts);
      }
      coasts[nCoasts] = new PShape();
      coasts[nCoasts] = createShape();
      coasts[nCoasts].beginShape();
    } else {
      PVector newLoc = new PVector( float(oneLine[0]), float(oneLine[1]) );
      if( coasts[nCoasts].getVertexCount() > epsilon ) {
        PVector prevLoc = coasts[nCoasts].getVertex(coasts[nCoasts].getVertexCount()-1);
        newLoc.sub(prevLoc);
        if( newLoc.magSq() > epsilon ) {
          coasts[nCoasts].vertex(float(oneLine[0]), float(oneLine[1]) );
        }
      } else {
        coasts[nCoasts].vertex(float(oneLine[0]), float(oneLine[1]) );
      }
    }
  }
  coasts[nCoasts].endShape();
  nCoasts++;
  println("Read " + nCoasts + " coasts");
  coasts = (PShape[])subset(coasts,0,nCoasts);
}

void readTrack(String fname) {
  String[] lines = loadStrings(fname);
  String[] oneLine;
  println("Reading "+fname+"  (read " + lines.length + " lines)");
  track = new PVector[lines.length];
  for( int i = 0; i < lines.length; i++ ) {
    oneLine = split(lines[i], ",");
    track[i] = new PVector(float(oneLine[0]), float(oneLine[1]));
  }
}


