
float calcX( float lati, float longi ) {
  float res = 0.0;
  longi = longiRange(longi + longioffset);
  /*** WINKEL TRIPEL ***/
  if( projectionType == WINKEL ) {
    if( projectionScale == SCALE_FREE ) {
      lati += latioffset;
    }
    float phi1 = HALF_PI; // changes "roundness"
    float alph = acos(cos(radians(lati))*cos((radians(longi))/2.0));
    res = 0.5*(radians(longi)*cos(radians(phi1))+2.0*cos(radians(lati))*sin(radians(longi)/2.0)/sinc(alph));
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -PI, PI, margin, width-margin);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, -HALF_PI, HALF_PI, margin, width-margin);
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res, -QUARTER_PI, QUARTER_PI, margin, width-margin);
    } else if( projectionScale == SCALE_FREE ) {
      res = map(res*scaleFactor, -PI, PI, margin, width-margin);
    }
  }
  /*** KAVRAYSKIY ***/
  else if( projectionType == KAVRAY ) {
    res = 3.0*radians(longi)/TWO_PI;
    res *= sqrt((PI*PI)/3.0 - radians(lati)*radians(lati));
    res *= rescale * twoOvrSqrt3;
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -PI, PI, margin, width-margin);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, -HALF_PI, HALF_PI, margin, width-margin);
    } else if( projectionScale == SCALE_LOWER48 ) {
      res = map(res, -QUARTER_PI, QUARTER_PI, margin, width-margin);
    } else if( projectionScale == SCALE_FREE ) {
      res = map(res*scaleFactor, -PI, PI, margin, width-margin);
    }
  }
  /*** MERCATOR ***/
  else if( projectionType == MERCATOR ) {
    res = radians(longi-0)*float(width)/TWO_PI+width/2;
    if( projectionScale == SCALE_GLOBE ) {
      // do nothing
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, 0.0*width, 0.55*width, 0, width); 
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res, 0.125*width, 0.35*width, 0, width); 
    }
  }
  /*** LAMBERT CYLINDRICAL EQ.AREA ***/
  else if( projectionType == LAMBERT_CYL_EA ) {
    if( projectionScale == SCALE_GLOBE ) {
      res = map(longi, -180, 180, 0, width);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      //res = map(longi, -80, 100, 0, width); 
      res = map(longi, -90, 90, 0, width); 
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(longi, -45, 45, 0, width); 
    }
  }
  /*** GALL STEREOGRAPHIC ***/
  else if( projectionType == GALL_STEREO ) {
    res = longi/sqrt(2.0);
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -180/sqrt(2.0), 180/sqrt(2.0), 0, width);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, -90/sqrt(2.0), 90/sqrt(2.0), 0, width);
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res, -45/sqrt(2.0), 45/sqrt(2.0), 0, width);
    }
  }
  /*** ALBERS CONIC EQ.AREA ***/
  else if( projectionType == ALBERS_CON_EA ) {
    // http://mathworld.wolfram.com/AlbersEqual-AreaConicProjection.html
    float lon0 = radians(0.0);
    float lat0 = radians(0.0);
    float lat1 = radians(0.0);
    float lat2 = radians(60.0);
    if(  projectionScale == SCALE_NORTHWEST ) {
      lat0 = radians(39.0);
      lat1 = radians(15.0);
      lat2 = radians(60.0);
    } else if( projectionScale == SCALE_LOWER48 ) {
      lat0 = radians(39.0);
      lat1 = radians(33.0);
      lat2 = radians(45.0);
    }
    float n = 0.5*(sin(lat1)+sin(lat2));
    float theta = n*(radians(longi)-lon0);
    float C = sq(cos(lat1)) + 2*n*sin(lat1);
    float rho = sqrt(C-2*n*sin(radians(lati)))/n;
    res = rho*sin(theta);
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -PI, PI, 0, width);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      //res = map(res, -1.5, 1.7, 0, width);
      res = map(res, -HALF_PI, HALF_PI, 0, width);
    } else if( projectionScale == SCALE_LOWER48 ) {
      res = map(res, -QUARTER_PI, QUARTER_PI, 0, width);
    }
  }
  return res;
}

float calcY( float lati, float longi ) {
  float res = 0.0;
  longi = longiRange(longi + longioffset);
  /*** WINKEL TRIPEL ***/
  if( projectionType == WINKEL ) {
    if( projectionScale == SCALE_FREE ) {
      lati += latioffset;
    }
    float alph = acos(cos(radians(lati))*cos((radians(longi))/2.0));
    res = 0.5*(radians(lati)+sin(radians(lati))/sinc(alph));
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -HALF_PI, HALF_PI, height-margin, margin);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, 0, HALF_PI, height-margin, margin);
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res, radians(12.5), radians(57.5), height-margin, margin);
    } else if( projectionScale == SCALE_FREE ) {
      res = map(res*scaleFactor, -HALF_PI, HALF_PI, height-margin, margin);
    }
  }
  /*** KAVRAYSKIY ***/
  else if( projectionType == KAVRAY ) {
    res = lati;
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -90, 90, height-margin, margin);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, 0, 90, height-margin, margin);
    } else if( projectionScale == SCALE_LOWER48 ) {
      res = map(res, 35-22.5, 35+22.5, height-margin, margin);
    } else if( projectionScale == SCALE_FREE ) {
      res = map((res+latioffset)*scaleFactor, -90, 90, height-margin, margin);
    }
  }
  /*** MERCATOR ***/
  else if( projectionType == MERCATOR ) {
    res = -(height/TWO_PI)*log(tan(QUARTER_PI+radians(lati)/2.0))+height/2;
    if( projectionScale == SCALE_GLOBE ) {
      // do nothing
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, 0.2*height, 0.55*height, 0, height);
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res, 0.325*height, 0.45*height, 0, height); 
    }
  }
  /*** LAMBERT CYLINDRICAL EQ.AREA ***/
  else if( projectionType == LAMBERT_CYL_EA ) {
    res = sin(radians(lati));
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -1, 1, height, 0);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      //res = map(res, 0, 1, height, 0);
      res = map(res, -0.05, 0.95, height, 0);
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res,  0.5736-0.25, 0.5736+0.25, height, 0);
    }
  }
  /*** GALL STEREOGRAPHIC ***/
  else if( projectionType == GALL_STEREO ) {
    float v = (1+sqrt(2.0)/2.0);
    res = v*tan(radians(lati/2.0));
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -v, v, height, 0);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      res = map(res, 0, v, height, 0);
    } else if( projectionScale == SCALE_LOWER48) {
      res = map(res, v*(35-22.5)/90.0, v*(35+22.5)/90.0, height, 0);
    }
  }
  /*** ALBERS CONIC EQ.AREA ***/
  else if( projectionType == ALBERS_CON_EA ) {
    // http://mathworld.wolfram.com/AlbersEqual-AreaConicProjection.html
    float lon0 = radians(0.0);
    float lat0 = radians(0.0);
    float lat1 = radians(0.0);
    float lat2 = radians(60.0);
    if(  projectionScale == SCALE_NORTHWEST ) {
      lat0 = radians(39.0);
      lat1 = radians(15.0);
      lat2 = radians(60.0);
    } else if( projectionScale == SCALE_LOWER48 ) {
      lat0 = radians(39.0);
      lat1 = radians(33.0);
      lat2 = radians(45.0);
    }
    float n = 0.5*(sin(lat1)+sin(lat2));
    float theta = n*(radians(longi)-lon0);
    float C = sq(cos(lat1)) + 2*n*sin(lat1);
    float rho = sqrt(C-2*n*sin(radians(lati)))/n;
    float rho0 = sqrt(C-2*n*sin(lat0))/n;
    res = rho0 - rho*cos(theta);
    if( projectionScale == SCALE_GLOBE ) {
      res = map(res, -0.8452995, 2.1328208, height, 0);
    } else if( projectionScale == SCALE_NORTHWEST ) {
      //res = map(res, -0.75, 0.85, height, 0);
      res = map(res, -0.75, 0.75, height, 0);
    } else if( projectionScale == SCALE_LOWER48 ) {
      res = map(res, -0.4, 0.3, height, 0);
    }
  }
  return res;
}

float longiRange(float longi) {
  if( longi > 180.0 ) {
    longi = longiRange(longi - 360.0);
  } else if( longi < -180.0 ) {
    longi = longiRange(longi + 360);
  }
  return longi;
}

/* these are part of the new code I'm adding to zoom and pan */
/* they check to see if a line segment will wrap around from 
     one day of the globe to the other */
boolean overTheLine( float x1, float y1, float x2, float y2 ) {
  return ( dist( longiRange(x1+longioffset), y1, longiRange(x2+longioffset), y2 ) > 10 );
}
boolean overTheLine( PVector v1, PVector v2 ) {
  return ( dist( longiRange(v1.x+longioffset), v1.y, longiRange(v2.x+longioffset), v2.y ) > 10 );
}

