/* drawMapLayer renders the borders, coasts, graticules, etc. */
void drawMapLayer() {
  pushStyle();

  /* map background */
  fill(239, 247, 255);          // "Alice Blue"
  stroke(30, 142, 255, 128);    // "Dodger Blue"
  beginShape();
  if( projectionType != MERCATOR ) {
    for ( float i = -90; i <= 90.0; i += 5.0 ) {
      /* left border, bottom to top */
      vertexLL( i, -180.0-longioffset );
    }
    for ( float i = -180; i <= 180; i += 5.0 ) {
      /* top border, left to right */
      vertexLL( 90, i-longioffset );
    }
    for ( float i = 90; i >= -90; i -= 5.0 ) {
      /* right border, top to bottom */
      vertexLL( i, 180-longioffset );
    }
    for ( float i = 180; i >= -180; i -= 5.0 ) {
      /* bottom border, right to left */
      vertexLL( -90, i-longioffset );
    }
  } else {
    /* Mercator can't project the poles onto the plane */
    /*    just draw a rectangle slightly inside the limits */
    vertexLL( -89.9999, -179.9999 );
    vertexLL(  89.9999, -179.9999 );
    vertexLL(  89.9999,  179.9999 );
    vertexLL( -89.9999,  179.9999 );
  }
  endShape(CLOSE);
  
  if( showStormTrack )  { renderTrack(); }
  if( showCoastline )   { renderCoasts(); }
  if( showIntlBorders ) { renderBorders(); }
  
  /* draw graticule */
  
  stroke(30, 142, 255);    // "Dodger Blue"
  float stepSize = 30.0;
  if ( projectionScale == SCALE_NORTHWEST ) {
    stepSize = 20.0;
  } 
  else if ( projectionScale == SCALE_LOWER48) {
    stepSize = 10.0;
  }
  /* draw lines of longitude */
  for ( int i = -180; i <= 180; i += stepSize ) {
    stroke(30, 142, 255, 128);
    longLine(i);
    if ( i < 180 ) {
      stroke(30, 142, 255, 64);
      longLine(i+stepSize/2.0);
    }
  }
  /* draw lines of lattitude */
  for ( int i = -90; i <= 90; i += stepSize ) {
    stroke(30, 142, 255, 128);
    latLine(i);
    if ( i < 90 ) {
      stroke(30, 142, 255, 64);
      latLine(i+stepSize/2.0);
    }
  }
  
  strokeWeight(1.5);
  /* tropics */
  latLineDashed(23.4378);
  latLineDashed(-23.4378);
  /* polar circles */
  latLineDashed(66.5622);
  latLineDashed(-66.5622);
  /* equator & prime meridian */
  latLine(0);
  longLine(0);
  
  popStyle();
}

void renderBorders() {
  pushStyle();
  PVector tmp = new PVector();
  stroke(92,128,164,128);  // "Air Force Blue"
  strokeWeight(1.0);
  noFill();
  for( int i = 0; i < nBorders; i++ ) {
    beginShape();
    vertexLL( borders[i].getVertex(0).y, borders[i].getVertex(0).x );
    for( int j = 1; j < borders[i].getVertexCount(); j++ ) {
      if( !overTheLine(borders[i].getVertex(j-1), borders[i].getVertex(j)) ) {
        vertexLL( borders[i].getVertex(j).y, borders[i].getVertex(j).x );
      } else {
        endShape();
        beginShape();
      }
    }
    endShape();
  }
  popStyle();
}

void renderCoasts() {
  pushStyle();
  PVector tmp = new PVector();
  stroke(98, 128, 148);  // "Horizon" blue-gray
  strokeWeight(1.25);
  noFill();
  for( int i = 0; i < nCoasts; i++ ) {
    beginShape();
    vertexLL( coasts[i].getVertex(0).y, coasts[i].getVertex(0).x );
    for( int j = 1; j < coasts[i].getVertexCount(); j++ ) {
      if( !overTheLine(coasts[i].getVertex(j-1), coasts[i].getVertex(j)) ) {
        vertexLL( coasts[i].getVertex(j).y, coasts[i].getVertex(j).x );
      } else {
        endShape();
        beginShape();
      }
    }
    endShape();
  }
  popStyle();
}

/* draw the path of the hurricane */
void renderTrack() {
  pushStyle();
  stroke(192,200,200);  // "Geyser" grey
  noFill();
  // NB don't use any alpha in the color here since the segments overlap
  float sw = 8;
  /* draw a wider path when the view is zoomed in */
  if( projectionScale == SCALE_GLOBE ) {
    sw = 4;
  } else if( projectionScale == SCALE_NORTHWEST ) {
    sw = 6;
  } else if( projectionScale == SCALE_LOWER48 ) {
    sw = 9;
  }
  strokeWeight(sw);
  strokeCap(ROUND);
  beginShape();
  vertexLL( track[0].x, track[0].y );
  for(int i = 1; i < track.length-1; i++ ) {
    if( !overTheLine(track[i-1].y, track[i-1].x, track[i].y, track[i].x ) ) {
      vertexLL( track[i].x, track[i].y );
    } else {
      endShape();
      beginShape();
    }
  }
  endShape();
  popStyle();
}

/* draw a line of longitude */
void longLine( float longi ) {
  int n = 256;
  float lat1, lat2;
  for ( int i = 0; i < n; i++ ) {
    lat1 = map(i, 0, n, -90, 90);
    lat2 = map(i+1, 0, n, -90, 90);
    lineLL( lat1, longi, lat2, longi );
  }
}

/* draw a line of latitude */
void latLine( float lati ) {
  int n = 256;
  float lon1, lon2;
  for ( int i = 0; i < n; i++ ) {
    lon1 = map(i, 0, n, -180-longioffset, 180-longioffset);
    lon2 = map(i+1, 0, n, -180-longioffset, 180-longioffset);
    lineLL( lati, lon1, lati, lon2 );
  }
}

/* draw a line of latitude, but dashed */
/*   (used for tropics, artic circle, etc. */
void latLineDashed( float lati ) {
  int nDashes = 201;
  // nDashes should be odd so the line begins and ends with a dash 
  if( projectionScale == SCALE_GLOBE ) {
    nDashes = 191;
  } else if( projectionScale == SCALE_NORTHWEST ) {
    nDashes = 251;
  } else if( projectionScale == SCALE_LOWER48 ) {
    nDashes = 401;
  }
  for ( int i = 0; i < nDashes; i+=2 ) {
    float lon1 = map(i, 0, nDashes, longiRange(-180), longiRange(180));
    float lon2 = map(i+1, 0, nDashes, longiRange(-180), longiRange(180));
    float x1 = calcX(lati, lon1);
    float y1 = calcY(lati, lon1);
    float x2 = calcX(lati, lon2);
    float y2 = calcY(lati, lon2);
    if( abs(x1-x2) < 100 ) {
      line( x1, y1, x2, y2 );
    }
  }
}

